@@ -7,10 +7,11 @@
 # Note: this dependency on Perl is fine: only developers use autogen.sh
 #       and we can state that dev people need Perl on their machine
 #
+# TODO: make sure that ac_pkg_swig.m4 gets installed in m4 directory ...
 
 rm -f autogen.err
 
-automake --version | perl -ne 'if (/\(GNU automake\) ([0-9].[0-9])/) {print;  if ($1 < 1.4) {exit 1;}}'
+automake --version | perl -ne 'if (/\(GNU automake\) (([0-9]+).([0-9]+))/) {print; if ($2 < 1 || ($2 == 1 && $3 < 4)) {exit 1;}}'
 
 if [ $? -ne 0 ]; then
     echo "Error: you need automake 1.4 or later.  Please upgrade."
@@ -73,7 +74,9 @@ for arg in $*; do
 done
 
 if $run_configure; then
-    ./configure --enable-maintainer-mode "$@"
+    mkdir -p build
+    cd build
+    ../configure --enable-maintainer-mode "$@"
     echo
     echo "Now type 'make' to compile link-grammar."
 else
