@@ -1,11 +1,14 @@
 #
-# High-level Python bindings build on top of the low-level
-# C API (clinkgrammar)
+# High-level Python bindings built on top of the low-level
+# C API (clinkgrammar).
 
 import locale
 
 import _clinkgrammar as clg
 
+__all__ = ['Parser', 'ParseOptions', 'Dictionary', 'Link', 'Linkage']
+
+
 class Parser(object):
     # A cost of 2.7 allows the usual cost-2 connectors, plus the
     # assorted fractional costs, without going to cost 3.0, which
