@@ -313,7 +313,7 @@ struct Postprocessor_s
  **********************************************************/
 
 /**
- * This is for building the graphs of links in post-processing.
+ * This summarizes the linkage status.
  */
 struct Linkage_info_struct
 {
@@ -325,12 +325,7 @@ struct Linkage_info_struct
 
 	double disjunct_cost;
 	double corpus_cost;
-	size_t nwords;
 	const char *pp_violation_msg;
-	char **disjunct_list_str;
-#ifdef USE_CORPUS
-	Sense **sense_list;
-#endif
 };
 
 /**
@@ -351,7 +346,7 @@ struct Linkage_info_struct
  */
 struct Linkage_s
 {
-	size_t          num_words;    /* Number of (tokenized) words */
+	WordIdx         num_words;    /* Number of (tokenized) words */
 	const char *  * word;         /* Array of word spellings */
 
 	size_t          num_links;    /* Number of links in array */
@@ -360,10 +355,13 @@ struct Linkage_s
 
 	Disjunct **     chosen_disjuncts; /* Disjuncts used, one per word */
 	size_t          cdsz;         /* Alloc'ed length of chosen_disjuncts */
+	char **         disjunct_list_str; /* Stringified version of above */
+#ifdef USE_CORPUS
+	Sense **        sense_list;   /* Word senses, infered from disjuncts */
+#endif
 
 	Linkage_info    lifo;         /* Parse_set index and cost information */
 	PP_info *       pp_info;      /* PP info, one for each link */
-	const char *    pp_violation; /* Name of violation, if any */
 	PP_data         hpsg_pp_data; /* Used in constituent code */
 };
 
