@@ -21,25 +21,20 @@
 
 #define MAX_TOKEN_LENGTH 250     /* Maximum number of chars in a token */
 
-/* MAX_SENTENCE cannot be more than 254, because word MAX_SENTENCE+1 is 
- * used to indicate that nothing can connect to this connector, and this 
- * should fit in one byte (if the word field of a connector is an 
- * unsigned char).
+/* MAX_SENTENCE cannot be more than 65534, because word MAX_SENTENCE+1 is
+ * used to indicate that nothing can connect to this connector, and this
+ * should fit in two bytes (because the word field of a connector is an
+ * unsigned short).
  */
-#define MAX_SENTENCE 250       /* Maximum number of words in a sentence */
-#define MAX_LINKS (2*MAX_SENTENCE-3) /* maximum number of links allowed */
+#define MAX_SENTENCE 65534      /* Maximum number of words in a sentence */
 
 /* Widely used private typedefs */
-#ifdef USE_FAT_LINKAGES
-typedef struct And_data_s And_data;
-typedef struct DIS_node_struct DIS_node;
-#endif /* USE_FAT_LINKAGES */
 typedef struct Connector_struct Connector;
 typedef struct Cost_Model_s Cost_Model;
 typedef struct Domain_s Domain;
 typedef struct DTreeLeaf_s DTreeLeaf;
+typedef struct Exp_list_s Exp_list;
 typedef struct Image_node_struct Image_node;
-typedef struct Label_node_s Label_node;
 typedef struct Linkage_info_struct Linkage_info;
 typedef struct Parse_info_struct *Parse_info;
 typedef struct Postprocessor_s Postprocessor;
@@ -47,14 +42,10 @@ typedef struct PP_data_s PP_data;
 typedef struct PP_info_s PP_info;
 typedef struct Regex_node_s Regex_node;
 typedef struct Resources_s * Resources;
-typedef struct Sublinkage_s Sublinkage;
 
 /* Some of the more obscure typedefs */
 typedef struct count_context_s count_context_t;
 typedef struct fast_matcher_s fast_matcher_t;
-#ifdef USE_FAT_LINKAGES
-typedef struct analyze_context_s analyze_context_t;
-#endif /* USE_FAT_LINKAGES */
 
 typedef struct Connector_set_s Connector_set;
 typedef struct Disjunct_struct Disjunct;
