@@ -16,9 +16,10 @@
 
 void build_sentence_disjuncts(Sentence sent, double cost_cutoff);
 X_node *   build_word_expressions(Dictionary dict, const char *);
-#ifdef OBSOLETE_MEMORY_PIGGY
-Disjunct * build_disjuncts_for_dict_node(Dict_node *);
-#endif
-Disjunct * build_disjuncts_for_X_node(X_node * x, double cost_cutoff);
+Disjunct * build_disjuncts_for_exp(Exp*, const char*, double cost_cutoff);
 
 unsigned int count_disjunct_for_dict_node(Dict_node *dn);
+
+#ifdef DEBUG
+void prt_exp(Exp *, int);
+#endif
