@@ -1,6 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright (c) 2014 Linas Vepstas                                      */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -16,19 +17,13 @@
 #include "api-structures.h"
 #include "error.h"
 #include "externs.h"
-#include "constituents.h"
 #include "post-process.h"
 #include "print-util.h"
 #include "string-set.h"
 
-#define MAXCONSTITUENTS 8192
 #define OPEN_BRACKET '['
 #define CLOSE_BRACKET ']'
 
-#ifdef USE_FAT_LINKAGES
-#define MAXSUBL 16
-#endif /* USE_FAT_LINKAGES */
-
 typedef enum {OPEN_TOK, CLOSE_TOK, WORD_TOK} CType;
 typedef enum {NONE, STYPE, PTYPE, QTYPE, QDTYPE} WType;
 
@@ -38,42 +33,35 @@ typedef struct
 	const char * start_link;
 	size_t left;      /* leftmost word */
 	size_t right;     /* rightmost word */
-#ifdef USE_FAT_LINKAGES
-	int subl;
-#endif /* USE_FAT_LINKAGES */
 	int canon;
 	bool valid;
 	char domain_type;
 } constituent_t;
 
-/* XXX it seems like the old code worked fine with MAX_ELTS=10 */
-#define MAX_ELTS 100
-typedef struct
-{
-	int num;
-	int e[MAX_ELTS];
-	bool valid;
-} andlist_t;
-
 /*
  * Context used to store assorted intermediate data
  * when the constituent string is being generated.
  */
-#define MAX_ANDS 1024
 typedef struct
 {
 	String_set * phrase_ss;
-	WType wordtype[MAX_SENTENCE];
-#ifdef USE_FAT_LINKAGES
-	bool word_used[MAXSUBL][MAX_SENTENCE];
-#else
-	bool word_used[MAX_SENTENCE];
-#endif /* USE_FAT_LINKAGES */
-	int templist[MAX_ELTS];
-	constituent_t constituent[MAXCONSTITUENTS];
-	andlist_t andlist[MAX_ANDS];
+	WType * wordtype;
+	constituent_t * constituent;
+	int conlen;
 } con_context_t;
 
+
+typedef struct CNode_s CNode;
+
+/* Invariant: Leaf if child==NULL */
+struct CNode_s
+{
+  char  * label;
+  CNode * child;
+  CNode * next;
+  int   start, end;
+};
+
 /* ================================================================ */
 
 static inline bool uppercompare(const char * s, const char * t)
@@ -83,25 +71,14 @@ static inline bool uppercompare(const char * s, const char * t)
 
 /**
  * If a constituent c has a comma at either end, we exclude the
- * comma. (We continue to shift the boundary until we get to
- * something inside the current sublinkage)
+ * comma.
  */
 static void adjust_for_left_comma(con_context_t * ctxt, Linkage linkage, int c)
 {
 	int w;
 	w = ctxt->constituent[c].left;
 	if (strcmp(linkage->word[w], ",") == 0)
-	{
 		w++;
-		while (1) {
-#ifdef USE_FAT_LINKAGES
-			if (ctxt->word_used[linkage->current][w]) break;
-#else
-			if (ctxt->word_used[w]) break;
-#endif /* USE_FAT_LINKAGES */
-			w++;
-		}
-	}
 	ctxt->constituent[c].left = w;
 }
 
@@ -113,15 +90,6 @@ static void adjust_for_right_comma(con_context_t *ctxt, Linkage linkage, int c)
 	    (strcmp(linkage->word[w], "RIGHT-WALL") == 0))
 	{
 		w--;
-		while (1)
-		{
-#ifdef USE_FAT_LINKAGES
-			if (ctxt->word_used[linkage->current][w]) break;
-#else
-			if (ctxt->word_used[w]) break;
-#endif /* USE_FAT_LINKAGES */
-			w--;
-		}
 	}
 	ctxt->constituent[c].right = w;
 }
@@ -141,9 +109,8 @@ static void print_constituent(con_context_t *ctxt, Linkage linkage, int c)
 }
 
 /******************************************************
- *        These functions do the bulk of the actual
- * constituent-generating; they're called once for each
- *                      sublinkage
+ * These functions do the bulk of the actual
+ * constituent-generating; they're called once.
  *********************************************************/
 
 typedef enum
@@ -259,23 +226,12 @@ static int gen_comp(con_context_t *ctxt, Linkage linkage,
 
 					/* if the new constituent (c) is to the left
 					   of c1, its right edge should be adjacent to the
-					   left edge of c1 - or as close as possible
-					   without going outside the current sublinkage.
-					   (Or substituting right and left as necessary.) */
-
-					if ((x==CASE_OPENER) || (x==CASE_PPOPEN) || (x==CASE_PART_OPEN)) {
+					   left edge of c1 - or as close as possible. */
+					if ((x==CASE_OPENER) || (x==CASE_PPOPEN) || (x==CASE_PART_OPEN))
+					{
 								/* This is the case where c is to the
 								   RIGHT of c1 */
-						w = ctxt->constituent[c1].right+1;
-						while(1) {
-#ifdef USE_FAT_LINKAGES
-							if (ctxt->word_used[linkage->current][w])
-#else
-							if (ctxt->word_used[w])
-#endif /* USE_FAT_LINKAGES */
-								break;
-							w++;
-						}
+						w = ctxt->constituent[c1].right + 1;
 						if (w > ctxt->constituent[c2].right)
 						{
 							done = true;
@@ -284,17 +240,9 @@ static int gen_comp(con_context_t *ctxt, Linkage linkage,
 						ctxt->constituent[c].left = w;
 						ctxt->constituent[c].right = ctxt->constituent[c2].right;
 					}
-					else {
-						w = ctxt->constituent[c1].left-1;
-						while(1) {
-#ifdef USE_FAT_LINKAGES
-							if (ctxt->word_used[linkage->current][w])
-#else
-							if (ctxt->word_used[w])
-#endif /* USE_FAT_LINKAGES */
-								break;
-							w--;
-						}
+					else
+					{
+						w = ctxt->constituent[c1].left - 1;
 						if (w < ctxt->constituent[c2].left) {
 							done = true;
 							continue;
@@ -319,13 +267,7 @@ static int gen_comp(con_context_t *ctxt, Linkage linkage,
 						print_constituent(ctxt, linkage, c);
 					}
 					c++;
-					if (MAXCONSTITUENTS <= c)
-					{
-						err_ctxt ec;
-						ec.sent = linkage->sent;
-						err_msg(&ec, Error, "Error: Too many constituents (a).\n");
-						c--;
-					}
+					assert (c < ctxt->conlen, "Too many constituents");
 					done = true;
 				}
 			}
@@ -376,15 +318,7 @@ static void adjust_subordinate_clauses(con_context_t *ctxt, Linkage linkage,
 					if ((ctxt->constituent[c2].domain_type == 'v') ||
 						(ctxt->constituent[c2].domain_type == 'a'))
 					{
-						w = ctxt->constituent[c].left-1;
-						while (true) {
-#ifdef USE_FAT_LINKAGES
-							if (ctxt->word_used[linkage->current][w]) break;
-#else
-							if (ctxt->word_used[w]) break;
-#endif /* USE_FAT_LINKAGES */
-							w--;
-						}
+						w = ctxt->constituent[c].left - 1;
 						ctxt->constituent[c2].right = w;
 
 						if (verbosity >= 2)
@@ -401,164 +335,22 @@ static void adjust_subordinate_clauses(con_context_t *ctxt, Linkage linkage,
 
 /******************************************************
  * These functions are called once, after constituents
- * for each sublinkage have been generated, to merge them
- * together and fix up some other things.
+ * have been generated, to merge them together and fix up
+ * some other things.
  *
  ********************************************************/
 
-/**
- * Here we're looking for the next andlist element to add on
- * to a conjectural andlist, stored in the array templist.
- * We go through the constituents, starting at "start".
- */
-static int find_next_element(con_context_t *ctxt,
-                             Linkage linkage,
-                             int start,
-                             int numcon_total,
-                             int num_elements,
-                             int num_lists)
-{
-	int a, addedone=0;
-#ifdef USE_FAT_LINKAGES
-	int ok, c, c2, c3;
-#endif /* USE_FAT_LINKAGES */
-
-	assert(num_elements <= MAX_ELTS, "Constutent element array overflow!\n");
-
-#ifdef USE_FAT_LINKAGES
-	for (c = start + 1; c < numcon_total; c++)
-	{
-		constituent_t *cc = &ctxt->constituent[c];
-
-		if (false == cc->valid)
-			continue;
-		if (strcmp(ctxt->constituent[ctxt->templist[0]].type, cc->type) != 0)
-			continue;
-
-		ok = 1;
-		/* We're considering adding constituent c to the andlist.
-		   If c is in the same sublinkage as one of the other andlist
-		   elements, don't add it. If it overlaps with one of the other
-		   constituents, don't add it. If there's a constituent
-		   identical to c that occurs in a sublinkage in which one of
-		   the other elements occurs, don't add it. */
-
-		for (a=0; a<num_elements; a++)
-		{
-			int t = ctxt->templist[a];
-			constituent_t *ct = &ctxt->constituent[t];
-
-			if (cc->subl == ct->subl)
-			{
-				ok = 0;
-				break;
-			}
-
-			/* It appears that none of this code matters when fat linkages
-			 * are disabled. There's probably lots more in this file that
-			 * isn't used when fat linkages are disabled... */ 
-			/* Check for overlapping intervals */
-			if (((cc->left < ct->left) && (cc->right > ct->left))
-				||
-				((cc->left < ct->right) && (cc->right > ct->right))
-				||
-				((cc->left > ct->left) && (cc->right < ct->right)))
-			{
-				ok = 0;
-				break;
-			}
-			for (c2=0; c2<numcon_total; c2++)
-			{
-				if (ctxt->constituent[c2].canon != cc->canon)
-					continue;
-				for (c3=0; c3<numcon_total; c3++)
-				{
-					if ((ctxt->constituent[c3].canon == ct->canon)
-						&& (ctxt->constituent[c3].subl == ctxt->constituent[c2].subl))
-					{
-						ok = 0;
-						break;
-					}
-				}
-				if (!ok)
-					break;
-			}
-			if (!ok)
-				break;
-		}
-
-		if (ok == 0) continue;
-
-		ctxt->templist[num_elements] = c;
-		addedone = 1;
-		num_lists = find_next_element(ctxt, linkage, c, numcon_total,
-		                              num_elements+1, num_lists);
-
-		/* Test for overlow of the and-list.
-		 * With the current parser, the following will cause an
-		 * overflow:
-		 *
-		 * I have not seen the grysbok, or the suni, or the dibitag, or
-		 * the lechwi, or the aoul, or the gerenuk, or the blaauwbok,
-		 * or the chevrotain, or lots of others, but who in the world
-		 * could guess what they were or what they looked like, judging
-		 * only from the names?
-		 */
-		if (MAX_ANDS <= num_lists)
-		{
-			err_ctxt ec;
-			ec.sent = linkage->sent;
-			err_msg(&ec, Error, "Error: Constituent overflowed andlist!\n");
-			return MAX_ANDS;
-		}
-	}
-#endif /* USE_FAT_LINKAGES */
-
-	if (addedone == 0 && num_elements > 1)
-	{
-		for (a = 0; a < num_elements; a++) {
-			ctxt->andlist[num_lists].e[a] = ctxt->templist[a];
-			ctxt->andlist[num_lists].num = num_elements;
-		}
-		num_lists++;
-	}
-	return num_lists;
-}
-
-/* When fat-links are finally removed, almost all of the code
- * for merge_constituents will evaporate away.
- */
 static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_total)
 {
-	int c1, c2=0, c3, ok, a, n, a2, n2, match, listmatch, a3;
-	int num_lists, num_elements;
-
-	for (c1=0; c1<numcon_total; c1++)
-	{
-		ctxt->constituent[c1].valid = true;
-
-		/* Find and invalidate any constituents with negative length */
-		if (ctxt->constituent[c1].right < ctxt->constituent[c1].left)
-		{
-			if (verbosity >= 2)
-			{
-				err_ctxt ec;
-				ec.sent = linkage->sent;
-				err_msg(&ec, Warn, 
-					"Warning: Constituent %d has negative length. Deleting it.\n", c1);
-			}
-			ctxt->constituent[c1].valid = false;
-		}
-		ctxt->constituent[c1].canon = c1;
-	}
+	int c1, c2=0;
 
 	/* First go through and give each constituent a canonical number
 	   (the index number of the lowest-numbered constituent
 	   identical to it) */
-
 	for (c1 = 0; c1 < numcon_total; c1++)
 	{
-		if (ctxt->constituent[c1].canon != c1) continue;
+		ctxt->constituent[c1].valid = true;
+		ctxt->constituent[c1].canon = c1;
 		for (c2 = c1 + 1; c2 < numcon_total; c2++)
 		{
 			if ((ctxt->constituent[c1].left == ctxt->constituent[c2].left) &&
@@ -570,58 +362,11 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 		}
 	}
 
-#ifdef USE_FAT_LINKAGES
-	/* If constituents A and B in different sublinkages X and Y
-	 * have one endpoint in common, but A is larger at the other end,
-	 * and B has no duplicate in X, then declare B invalid. (Example:
-	 * " [A [B We saw the cat B] and the dog A] "
-	 */
-	for (c1 = 0; c1 < numcon_total; c1++)
-	{
-		if (false == ctxt->constituent[c1].valid) continue;
-		for (c2 = 0; c2 < numcon_total; c2++)
-		{
-			if (ctxt->constituent[c2].subl == ctxt->constituent[c1].subl) continue;
-			ok = 1;
-			/* Does c2 have a duplicate in the sublinkage containing c1?
-			   If so, bag it */
-			for (c3 = 0; c3 < numcon_total; c3++)
-			{
-				if ((ctxt->constituent[c2].canon == ctxt->constituent[c3].canon) &&
-					(ctxt->constituent[c3].subl == ctxt->constituent[c1].subl))
-					ok = 0;
-			}
-			for (c3 = 0; c3 < numcon_total; c3++)
-			{
-				if ((ctxt->constituent[c1].canon == ctxt->constituent[c3].canon) &&
-					(ctxt->constituent[c3].subl == ctxt->constituent[c2].subl))
-					ok = 0;
-			}
-			if (ok == 0) continue;
-			if ((ctxt->constituent[c1].left == ctxt->constituent[c2].left) &&
-				(ctxt->constituent[c1].right > ctxt->constituent[c2].right) &&
-				(strcmp(ctxt->constituent[c1].type, ctxt->constituent[c2].type) == 0))
-			{
-				ctxt->constituent[c2].valid = false;
-			}
-
-			if ((ctxt->constituent[c1].left < ctxt->constituent[c2].left) &&
-				(ctxt->constituent[c1].right == ctxt->constituent[c2].right) &&
-				(strcmp(ctxt->constituent[c1].type, ctxt->constituent[c2].type) == 0))
-			{
-				ctxt->constituent[c2].valid = false;
-			}
-		}
-	}
-#endif /* USE_FAT_LINKAGES */
-
 	/* Now go through and find duplicates; if a pair is found,
-	 * mark one as invalid. (It doesn't matter if they're in the
-	 * same sublinkage or not)
+	 * mark one as invalid.
 	 */
 	for (c1 = 0; c1 < numcon_total; c1++)
 	{
-		if (false == ctxt->constituent[c1].valid) continue;
 		for (c2 = c1 + 1; c2 < numcon_total; c2++)
 		{
 			if (ctxt->constituent[c2].canon == ctxt->constituent[c1].canon)
@@ -629,172 +374,7 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 		}
 	}
 
-	/* Now we generate the and-lists. An and-list is a set of mutually
-	 * exclusive constituents. Each constituent in the list may not
-	 * be present in the same sublinkage as any of the others.
-	 */
-	num_lists = 0;
-	for (c1 = 0; c1 < numcon_total; c1++)
-	{
-		if (ctxt->constituent[c1].valid == false) continue;
-		num_elements = 1;
-		ctxt->templist[0] = c1;
-		num_lists = find_next_element(ctxt, linkage, c1, numcon_total,
-			                           num_elements, num_lists);
-
-		/* If we're overflowing, then punt */
-		if (MAX_ANDS <= num_lists)
-			break;
-	}
-
-	if (verbosity >= 2)
-	{
-		printf("And-lists:\n");
-		for (n=0; n<num_lists; n++)
-		{
-			printf("  %d: ", n);
-			for (a=0; a < ctxt->andlist[n].num; a++)
-			{
-				printf("%d ", ctxt->andlist[n].e[a]);
-			}
-			printf("\n");
-		}
-	}
-
-	/* Now we prune out any andlists that are subsumed by other
-	 * andlists--e.g. if andlist X contains constituents A and B,
-	 * and Y contains A B and C, we throw out X
-	 */
-	for (n = 0; n < num_lists; n++)
-	{
-		ctxt->andlist[n].valid = true;
-		for (n2 = 0; n2 < num_lists; n2++)
-		{
-			if (n2 == n) continue;
-			if (ctxt->andlist[n2].num < ctxt->andlist[n].num)
-				continue;
-
-			listmatch = 1;
-			for (a = 0; a < ctxt->andlist[n].num; a++)
-			{
-				match = 0;
-				for (a2 = 0; a2 < ctxt->andlist[n2].num; a2++)
-				{
-					if (ctxt->andlist[n2].e[a2] == ctxt->andlist[n].e[a])
-						match = 1;
-				}
-				if (match == 0) listmatch = 0;
-				/* At least one element was not matched by n2 */
-			}
-			if (listmatch == 1) ctxt->andlist[n].valid = false;
-		}
-	}
-
-	/* If an element of an andlist contains an element of another
-	 * andlist, it must contain the entire andlist.
-	 */
-	for (n = 0; n < num_lists; n++)
-	{
-		if (ctxt->andlist[n].valid == false)
-			continue;
-		for (a = 0; (a < ctxt->andlist[n].num) && (ctxt->andlist[n].valid); a++)
-		{
-			for (n2 = 0; (n2 < num_lists) && (ctxt->andlist[n].valid); n2++)
-			{
-				if ((n2 == n) || (ctxt->andlist[n2].valid == false))
-					continue;
-				for (a2 = 0; (a2 < ctxt->andlist[n2].num) && (ctxt->andlist[n].valid); a2++)
-				{
-					c1 = ctxt->andlist[n].e[a];
-					c2 = ctxt->andlist[n2].e[a2];
-					if (c1 == c2)
-						continue;
-					if (!((ctxt->constituent[c2].left <= ctxt->constituent[c1].left) &&
-						  (ctxt->constituent[c2].right >= ctxt->constituent[c1].right)))
-						continue;
-					if (verbosity >= 2)
-						printf("Found that c%d in list %d is bigger " \
-							   "than c%d in list %d\n", c2, n2, c1, n);
-					ok = 1;
-
-					/* An element of n2 contains an element of n.
-					 * Now, we check to see if that element of n2
-					 * contains ALL the elements of n.
-					 * If not, n is invalid.
-					 */
-					for (a3 = 0; a3 < ctxt->andlist[n].num; a3++)
-					{
-						c3 = ctxt->andlist[n].e[a3];
-						if ((ctxt->constituent[c2].left>ctxt->constituent[c3].left) ||
-							(ctxt->constituent[c2].right<ctxt->constituent[c3].right))
-							ok = 0;
-					}
-					if (ok != 0)
-						continue;
-					ctxt->andlist[n].valid = false;
-					if (verbosity >= 2)
-					{
-						printf("Eliminating andlist, " \
-							   "n=%d, a=%d, n2=%d, a2=%d: ",
-							   n, a, n2, a2);
-						for (a3 = 0; a3 < ctxt->andlist[n].num; a3++)
-						{
-							printf("%d ", ctxt->andlist[n].e[a3]);
-						}
-						printf("\n");
-					}
-				}
-			}
-		}
-	}
-
-	if (verbosity >= 2)
-	{
-		printf("And-lists after pruning:\n");
-		for (n=0; n<num_lists; n++) {
-			if (false == ctxt->andlist[n].valid)
-				continue;
-			printf("  %d: ", n);
-			for (a=0; a<ctxt->andlist[n].num; a++) {
-				printf("%d ", ctxt->andlist[n].e[a]);
-			}
-			printf("\n");
-		}
-	}
-
-	c1 = numcon_total;
-	for (n = 0; n < num_lists; n++)
-	{
-		size_t leftend, rightend;
-		if (false == ctxt->andlist[n].valid) continue;
-		leftend = SIZE_MAX;
-		rightend = 0;
-		for (a = 0; a < ctxt->andlist[n].num; a++)
-		{
-			c2 = ctxt->andlist[n].e[a];
-			if (ctxt->constituent[c2].left < leftend)
-			{
-				leftend = ctxt->constituent[c2].left;
-			}
-			if (ctxt->constituent[c2].right > rightend)
-			{
-				rightend = ctxt->constituent[c2].right;
-			}
-		}
-
-		ctxt->constituent[c1].left = leftend;
-		ctxt->constituent[c1].right = rightend;
-		ctxt->constituent[c1].type = ctxt->constituent[c2].type;
-		ctxt->constituent[c1].domain_type = 'x';
-		ctxt->constituent[c1].valid = true;
-		ctxt->constituent[c1].start_link = ctxt->constituent[c2].start_link;  /* bogus */
-
-		if (verbosity >= 2)
-			printf("Adding constituent:\n");
-		print_constituent(ctxt, linkage, c1);
-		c1++;
-	}
-	return c1;
+	return numcon_total;
 }
 
 /**
@@ -803,7 +383,7 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
  * Pg*b, I, PP, or Pv, wordtype[w]=PTYPE. If it's a question-word
  * used in an indirect question, wordtype[w]=QTYPE. If it's a
  * question-word determiner,  wordtype[w]=QDTYPE. Else wordtype[w]=NONE.
- * (This function is called once for each sublinkage.)
+ * (This function is called once.)
  */
 static void generate_misc_word_info(con_context_t * ctxt, Linkage linkage)
 {
@@ -811,7 +391,7 @@ static void generate_misc_word_info(con_context_t * ctxt, Linkage linkage)
 	const char * label1, * label2;
 
 	for (w1 = 0; w1 < linkage->num_words; w1++)
-		ctxt->wordtype[w1]=NONE;
+		ctxt->wordtype[w1] = NONE;
 
 	for (l1 = 0; l1 < linkage_get_num_links(linkage); l1++) {	
 		w1=linkage_get_link_rword(linkage, l1);
@@ -855,7 +435,7 @@ static int new_style_conjunctions(con_context_t *ctxt, Linkage linkage, int numc
 	for (c = 0; c < numcon_total; c++)
 	{
 		constituent_t *ct = &ctxt->constituent[c];
-		printf("ola %d valid=%d %s start=%s lr=%d %d\n", c, 
+		lgdebug(3, "ola %d valid=%d %s start=%s lr=%zu %zu\n", c, 
 			ct->valid, ct->type, ct->start_link, ct->left, ct->right);
 	}
 #endif
@@ -864,12 +444,9 @@ static int new_style_conjunctions(con_context_t *ctxt, Linkage linkage, int numc
 
 static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_total)
 {
-	int c, c2;
+	int c;
 	bool global_leftend_found, global_rightend_found;
-	int newcon_total = 0;
 	size_t lastword;
-	Sentence sent;
-	sent = linkage_get_sentence(linkage);
 
 	for (c = 0; c < numcon_total; c++)
 	{
@@ -941,18 +518,29 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 		}
 	}
 
-	numcon_total += newcon_total;
-
 	/* If there's a global S constituent that includes everything
-	   except a final period or question mark, extend it by one word */
-
+	   except a final terminating punctuation (period or question mark),
+	   extend it by one word. We know its the terminating punctuation,
+	   because it links to the right wall with an RW link.  If its
+	   not, then that final link is not there...
+	 */
 	for (c = 0; c < numcon_total; c++)
 	{
-		if ((ctxt->constituent[c].right == linkage->num_words -3) &&
+		if ((ctxt->constituent[c].right == linkage->num_words - 3) &&
 			(ctxt->constituent[c].left == 1) &&
-			(strcmp(ctxt->constituent[c].type, "S") == 0) &&
-			(strcmp(sent->word[linkage->num_words -2].alternatives[0], ".") == 0))
-			ctxt->constituent[c].right++;
+			(strcmp(ctxt->constituent[c].type, "S") == 0))
+		{
+			size_t ln;
+			for (ln = 0; ln < linkage->num_links; ln++)
+			{
+				if ((linkage->link_array[ln].lw == linkage->num_words - 2) && 
+				    (linkage->link_array[ln].rw == linkage->num_words - 1))
+				{
+					ctxt->constituent[c].right++;
+					break;
+				}
+			}
+		}
 	}
 
 	/* If there's no S boundary at the very left end of the sentence,
@@ -970,6 +558,7 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 			global_leftend_found = true;
 		}
 	}
+
 	for (c = 0; c < numcon_total; c++)
 	{
 		if ((ctxt->constituent[c].right >= lastword) &&
@@ -979,6 +568,7 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 			global_rightend_found = true;
 		}
 	}
+
 	if ((global_leftend_found == false) || (global_rightend_found == false))
 	{
 		c = numcon_total;
@@ -993,92 +583,9 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 		print_constituent(ctxt, linkage, c);
 	}
 
-	/* Check once more to see if constituents are nested (checking 
-	 * BETWEEN sublinkages this time). This code probably goes away
-	 * when fat-links get removed. */
-	for (c = 0; c < numcon_total; c++)
-	{
-		if (false == ctxt->constituent[c].valid) continue;
-		for (c2 = 0; c2 < numcon_total; c2++)
-		{
-			if (false == ctxt->constituent[c2].valid) continue;
-			if ((ctxt->constituent[c].left < ctxt->constituent[c2].left) &&
-				(ctxt->constituent[c].right < ctxt->constituent[c2].right) &&
-				(ctxt->constituent[c].right >= ctxt->constituent[c2].left))
-			{
-				if (verbosity >= 2)
-				{
-					err_ctxt ec;
-					ec.sent = linkage->sent;
-					err_msg(&ec, Warn, "Warning: the constituents aren't nested! "
-					          "Adjusting them. (%d, %d)\n", c, c2);
-				}
-				ctxt->constituent[c].left = ctxt->constituent[c2].left;
-			}
-		}
-	}
 	return numcon_total;
 }
 
-/**
- * This function generates a table, word_used[i][w], showing
- * whether each word w is used in each sublinkage i; if so,
- * the value for that cell of the table is 1.
- */
-static void count_words_used(con_context_t *ctxt, Linkage linkage)
-{
-	size_t w, link;
-#ifdef USE_FAT_LINKAGES
-	int i, num_subl;
-
-	num_subl = linkage->num_sublinkages;
-	if (linkage->unionized == 1 && num_subl > 1) num_subl--;
-
-	if (verbosity >= 2)
-		printf("Number of sublinkages = %d\n", num_subl);
-
-	for (i=0; i<num_subl; i++)
-	{
-		for (w = 0; w < linkage->num_words; w++) ctxt->word_used[i][w] = false;
-		linkage->current = i;
-
-		for (link = 0; link < linkage_get_num_links(linkage); link++)
-		{
-			ctxt->word_used[i][linkage_get_link_lword(linkage, link)] = true;
-			ctxt->word_used[i][linkage_get_link_rword(linkage, link)] = true;
-		}
-		if (verbosity >= 2)
-		{
-			printf("Sublinkage %d: ", i);
-			for (w = 0; w < linkage->num_words; w++)
-			{
-				if (ctxt->word_used[i][w]) printf("1 ");
-				else printf("0 ");
-			}
-			printf("\n");
-		}
-	}
-#else
-	for (w = 0; w < linkage->num_words; w++) ctxt->word_used[w] = false;
-
-	for (link = 0; link < linkage_get_num_links(linkage); link++)
-	{
-		ctxt->word_used[linkage_get_link_lword(linkage, link)] = true;
-		ctxt->word_used[linkage_get_link_rword(linkage, link)] = true;
-	}
-	if (verbosity >= 2)
-	{
-		printf("Word used: ");
-		for (w = 0; w < linkage->num_words; w++)
-		{
-			if (ctxt->word_used[w]) printf("1 ");
-			else printf("0 ");
-		}
-		printf("\n");
-	}
-#endif /* USE_FAT_LINKAGES */
-}
-
 static int add_constituent(con_context_t *ctxt, int c, const Linkage linkage,
                            const Domain *domain,
                            int l, int r, const char * name)
@@ -1145,7 +652,6 @@ static const char * cons_of_domain(const Linkage linkage, char domain_type)
 	default:
 		{
 			err_ctxt ec;
-			ec.sent = linkage->sent;
 			err_msg(&ec, Error, "Error: Illegal domain: %c\n", domain_type);
 			return "";
 		}
@@ -1153,30 +659,19 @@ static const char * cons_of_domain(const Linkage linkage, char domain_type)
 }
 
 static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
-                                          int numcon_total
-#ifdef USE_FAT_LINKAGES
-                                          , int s
-#endif /* USE_FAT_LINKAGES */
-                                          )
+                                          int numcon_total)
 {
 	size_t d, l, w2;
-	size_t leftmost, rightmost, leftlimit;
 	int c, w, c2, numcon_subl = 0;
-	List_o_links * dlink;
-	int rootleft;
-	Sublinkage * subl;
-	const char * name = "";
-	Domain domain;
 
-#ifdef USE_FAT_LINKAGES
-	subl = &linkage->sublinkage[s];
-#else
-	subl = &linkage->sublinkage;
-#endif /* USE_FAT_LINKAGES */
-
-	for (d = 0, c = numcon_total; d < subl->pp_data.N_domains; d++, c++)
+	for (d = 0, c = numcon_total; d < linkage->hpsg_pp_data.N_domains; d++, c++)
 	{
-		domain = subl->pp_data.domain_array[d];
+		size_t leftmost, rightmost, leftlimit;
+		int rootleft;
+		List_o_links * dlink;
+
+		Domain domain = linkage->hpsg_pp_data.domain_array[d];
+
 		// rootright = linkage_get_link_rword(linkage, domain.start_link);
 		rootleft =  linkage_get_link_lword(linkage, domain.start_link);
 
@@ -1283,6 +778,7 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 			(post_process_match("Mr", ctxt->constituent[c].start_link) == 1) ||
 			(post_process_match("MX#d", ctxt->constituent[c].start_link) == 1))
 		{
+			const char * name = "";
 			w = leftmost;
 			if (strcmp(linkage->word[w], ",") == 0) w++;
 			if (ctxt->wordtype[w] == NONE)
@@ -1312,13 +808,11 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 		if (ctxt->constituent[c].domain_type == '\0')
 		{
 			err_ctxt ec;
-			ec.sent = linkage->sent;
 			err_msg(&ec, Error, "Error: no domain type assigned to constituent\n");
 		}
 		if (ctxt->constituent[c].start_link == NULL)
 		{
 			err_ctxt ec;
-			ec.sent = linkage->sent;
 			err_msg(&ec, Error, "Error: no type assigned to constituent\n");
 		}
 	}
@@ -1327,12 +821,7 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 	/* numcon_subl = handle_islands(linkage, numcon_total, numcon_subl);  */
 
 	if (verbosity >= 2)
-#ifdef USE_FAT_LINKAGES
-		printf("Constituents added at first stage for subl %d:\n",
-			   linkage->current);
-#else
 		printf("Constituents added at first stage:\n");
-#endif /* USE_FAT_LINKAGES */
 
 	for (c = numcon_total; c < numcon_total + numcon_subl; c++)
 	{
@@ -1429,7 +918,6 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 						if (verbosity >= 2)
 						{
 							err_ctxt ec;
-							ec.sent = linkage->sent;
 							err_msg(&ec, Warn, 
 							      "Warning: the constituents aren't nested! "
 							      "Adjusting them. (%d, %d)\n", c, c2);
@@ -1442,20 +930,7 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 		if (adjustment_made == false) break;
 	}
 
-	if (MAXCONSTITUENTS <= numcon_total + numcon_subl)
-	{
-		err_ctxt ec;
-		ec.sent = linkage->sent;
-		err_msg(&ec, Error, "Error: Too many constituents (a2).\n");
-		numcon_total = MAXCONSTITUENTS - numcon_subl;
-	}
-#ifdef USE_FAT_LINKAGES
-	for (c = numcon_total; c < numcon_total + numcon_subl; c++)
-	{
-		ctxt->constituent[c].subl = linkage->current;
-	}
-#endif /* USE_FAT_LINKAGES */
-
+	assert (numcon_total + numcon_subl < ctxt->conlen, "Too many constituents");
 	return numcon_subl;
 }
 
@@ -1465,15 +940,14 @@ exprint_constituent_structure(con_context_t *ctxt,
 {
 	size_t w;
 	int c;
-	bool leftdone[MAXCONSTITUENTS];
-	bool rightdone[MAXCONSTITUENTS];
+	bool *leftdone = alloca(numcon_total * sizeof(bool));
+	bool *rightdone = alloca(numcon_total * sizeof(bool));
 	int best, bestright, bestleft;
-	Sentence sent;
-	char s[MAX_WORD], *p;
+	char *p;
+	char s[MAX_WORD];
 	String * cs = string_new();
 
-	assert (numcon_total < MAXCONSTITUENTS, "Too many constituents (b)");
-	sent = linkage_get_sentence(linkage);
+	assert (numcon_total < ctxt->conlen, "Too many constituents (b)");
 
 	for (c = 0; c < numcon_total; c++)
 	{
@@ -1537,10 +1011,6 @@ exprint_constituent_structure(con_context_t *ctxt,
 				p = strchr(p, CLOSE_BRACKET);
 			}
 
-			/* Now, if the first character of the word was
-			   originally uppercase, we put it back that way */
-			if (sent->word[w].firstupper)
-				upcase_utf8_str(s, s, MAX_WORD);
 			append_string(cs, "%s ", s);
 		}
 
@@ -1572,82 +1042,23 @@ exprint_constituent_structure(con_context_t *ctxt,
 
 static char * do_print_flat_constituents(con_context_t *ctxt, Linkage linkage)
 {
-	Postprocessor * pp;
 	int numcon_total, numcon_subl;
 	char * q;
-#ifdef USE_FAT_LINKAGES
-	int s, num_subl;
-#endif /* USE_FAT_LINKAGES */
 
 	ctxt->phrase_ss = string_set_create();
-	pp = linkage->sent->dict->constituent_pp;
 	numcon_total = 0;
 
-	count_words_used(ctxt, linkage);
+	generate_misc_word_info(ctxt, linkage);
 
-#ifdef USE_FAT_LINKAGES
-	/* 1 < num_sublinkages only if a parse used fat links. */
-	/* A lot of the complixity here could go away once we
-	 * eliminate fat links for good ...  */
-	num_subl = linkage->num_sublinkages;
-	if (num_subl > MAXSUBL)
-	{
-	  num_subl = MAXSUBL;
-	  if (verbosity >= 2)
-		printf("Number of sublinkages exceeds maximum: only considering first %d sublinkages\n", MAXSUBL);
-	}
-
-	if (linkage->unionized == 1 && num_subl > 1) num_subl--;
-	for (s = 0; s < num_subl; s++)
-#endif /* USE_FAT_LINKAGES */
-	{
-#ifdef USE_FAT_LINKAGES
-		linkage_set_current_sublinkage(linkage, s);
-#endif /* USE_FAT_LINKAGES */
-		linkage_post_process(linkage, pp);
-		generate_misc_word_info(ctxt, linkage);
-
-#ifdef USE_FAT_LINKAGES
-		numcon_subl = read_constituents_from_domains(ctxt, linkage, numcon_total, s);
-#else
-		numcon_subl = read_constituents_from_domains(ctxt, linkage, numcon_total);
-#endif /* USE_FAT_LINKAGES */
-		numcon_total += numcon_subl;
-		if (MAXCONSTITUENTS <= numcon_total)
-		{
-			err_ctxt ec;
-			ec.sent = linkage->sent;
-			err_msg(&ec, Error, "Error: Too many constituents (c).\n");
-			numcon_total = MAXCONSTITUENTS-1;
-#ifdef USE_FAT_LINKAGES
-			break;
-#endif /* USE_FAT_LINKAGES */
-		}
-	}
+	numcon_subl = read_constituents_from_domains(ctxt, linkage, numcon_total);
+	numcon_total += numcon_subl;
+	assert (numcon_total < ctxt->conlen, "Too many constituents (c)");
 	numcon_total = merge_constituents(ctxt, linkage, numcon_total);
-	if (MAXCONSTITUENTS <= numcon_total)
-	{
-		err_ctxt ec;
-		ec.sent = linkage->sent;
-		err_msg(&ec, Error, "Error: Too many constituents (d).\n");
-		numcon_total = MAXCONSTITUENTS-1;
-	}
+	assert (numcon_total < ctxt->conlen, "Too many constituents (d)");
 	numcon_total = new_style_conjunctions(ctxt, linkage, numcon_total);
-	if (MAXCONSTITUENTS <= numcon_total)
-	{
-		err_ctxt ec;
-		ec.sent = linkage->sent;
-		err_msg(&ec, Error, "Error: Too many constituents (n).\n");
-		numcon_total = MAXCONSTITUENTS-1;
-	}
+	assert (numcon_total < ctxt->conlen, "Too many constituents (e)");
 	numcon_total = last_minute_fixes(ctxt, linkage, numcon_total);
-	if (MAXCONSTITUENTS <= numcon_total)
-	{
-		err_ctxt ec;
-		ec.sent = linkage->sent;
-		err_msg(&ec, Error, "Error: Too many constituents (e).\n");
-		numcon_total = MAXCONSTITUENTS-1;
-	}
+	assert (numcon_total < ctxt->conlen, "Too many constituents (f)");
 	q = exprint_constituent_structure(ctxt, linkage, numcon_total);
 	string_set_delete(ctxt->phrase_ss);
 	ctxt->phrase_ss = NULL;
@@ -1656,18 +1067,18 @@ static char * do_print_flat_constituents(con_context_t *ctxt, Linkage linkage)
 
 static char * print_flat_constituents(Linkage linkage)
 {
-	/* In principle, the ctxt could be allocated on stack, instead of
-	 * with malloc(). However, The java6 jvm (and MS Windows jvm's)
-	 * gives JNI clients only a small amount of stack space. Alloc'ing
-	 * this (rather large) structure  on stack will blow up the JVM.
-	 * This was discovered only after much work. Bummer.
-	 */
-	char * p;
-	con_context_t *ctxt = (con_context_t *) xalloc (sizeof(con_context_t));
+	size_t wts = linkage->num_words * sizeof(WType);
+	size_t cns = (linkage->num_links + linkage->num_words) * sizeof(constituent_t);
+
+	con_context_t *ctxt = (con_context_t *) alloca(sizeof(con_context_t));
 	memset(ctxt, 0, sizeof(con_context_t));
-	p = do_print_flat_constituents(ctxt, linkage);
-	xfree(ctxt, sizeof(con_context_t));
-	return p;
+	ctxt->wordtype = (WType *) alloca(wts);
+	memset(ctxt->wordtype, 0, wts);
+	ctxt->conlen = linkage->num_links + linkage->num_words;
+	ctxt->constituent = (constituent_t *) alloca(cns);
+	memset(ctxt->constituent, 0, cns);
+
+	return do_print_flat_constituents(ctxt, linkage);
 }
 
 static CType token_type (char *token)
@@ -1812,7 +1223,7 @@ static int assign_spans(CNode * n, int start)
 	return num_words;
 }
 
-CNode * linkage_constituent_tree(Linkage linkage)
+static CNode * linkage_constituent_tree(Linkage linkage)
 {
 	char *p, *q, *saveptr;
 	int len;
@@ -1834,7 +1245,7 @@ CNode * linkage_constituent_tree(Linkage linkage)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 
-void linkage_free_constituent_tree(CNode * n)
+static void linkage_free_constituent_tree(CNode * n)
 {
 	CNode *m, *x;
 	for (m=n->child; m!=NULL; m=x) {
@@ -1858,7 +1269,7 @@ char * linkage_print_constituent_tree(Linkage linkage, ConstituentDisplayStyle m
 	char * p;
 
 	if (!linkage) return NULL;
-	if ((mode == NO_DISPLAY) || (linkage->sent->dict->constituent_pp == NULL))
+	if (mode == NO_DISPLAY)
 	{
 		return NULL;
 	}
@@ -1886,28 +1297,3 @@ void linkage_free_constituent_tree_str(char * s)
 	exfree(s, strlen(s)+1);
 }
 
-const char * linkage_constituent_node_get_label(const CNode *n)
-{
-	return n->label;
-}
-
-
-CNode * linkage_constituent_node_get_child(const CNode *n)
-{
-	return n->child;
-}
-
-CNode * linkage_constituent_node_get_next(const CNode *n)
-{
-	return n->next;
-}
-
-int linkage_constituent_node_get_start(const CNode *n)
-{
-	return n->start;
-}
-
-int linkage_constituent_node_get_end(const CNode *n)
-{
-	return n->end;
-}
