@@ -21,13 +21,17 @@
  * They are not part of the public API. */
 
 bool find_word_in_dict(Dictionary dict, const char *);
-Afdict_class * afdict_find(Dictionary afdict, const char * con, bool notify_err);
+Afdict_class * afdict_find(Dictionary, const char *, bool);
 
-Exp * Exp_create(Dictionary);
-void add_empty_word(Dictionary, Dict_node *);
+Exp * Exp_create(Exp_list *);
+Exp * add_empty_word(Dictionary const, Exp_list *, Dict_node *);
+void free_Exp_list(Exp_list *);
 
 void patch_subscript(char *);
 
+bool is_suffix(const char, const char *);
+bool is_stem(const char *);
+
 /* Connector names for the affix class lists in the affix file */
 
 typedef enum {
@@ -41,15 +45,15 @@ typedef enum {
 	AFDICT_BULLETS,
 	AFDICT_INFIXMARK,
 	AFDICT_STEMSUBSCR,
-	AFDICT_SANEMORPHISM
+	AFDICT_SANEMORPHISM,
 #ifdef USE_ANYSPLIT
-	,
 	AFDICT_REGPRE,
 	AFDICT_REGMID,
 	AFDICT_REGSUF,
-	AFDICT_REGALTS ,
-	AFDICT_REGPARTS
+	AFDICT_REGALTS,
+	AFDICT_REGPARTS,
 #endif
+	AFDICT_NUM_ENTRIES
 } afdict_classnum;
 
 #define AFDICT_CLASSNAMES1 \
@@ -64,21 +68,20 @@ typedef enum {
 	"BULLETS", \
 	"INFIXMARK",   /* Prepended to suffixes, appended to pefixes */ \
 	"STEMSUBSCR",  /* Subscripts for stems */ \
-	"SANEMORPHISM" /* Regex for sane_morphism() */
+	"SANEMORPHISM", /* Regex for sane_morphism() */
 
 #ifdef USE_ANYSPLIT
 #define AFDICT_CLASSNAMES2 \
-	, \
 	"REGPRE",      /* Regex for prefix */ \
 	"REGMID",      /* Regex for middle parts */ \
 	"REGSUF",      /* Regex for suffix */ \
 	"REGALTS",     /* Min&max number of alternatives to issue for a word */\
-	"REGPARTS"     /* Max number of word partitions */
+	"REGPARTS",    /* Max number of word partitions */
 #else
 #define AFDICT_CLASSNAMES2
 #endif
 
-#define AFDICT_CLASSNAMES AFDICT_CLASSNAMES1 AFDICT_CLASSNAMES2
+#define AFDICT_CLASSNAMES AFDICT_CLASSNAMES1 AFDICT_CLASSNAMES2 "last classname"
 #define AFCLASS(afdict, class) (&afdict->afdict_class[class])
 
 #endif /* _LG_DICT_COMMON_H_ */
