@@ -462,9 +462,12 @@ dictionary_six_str(const char * lang,
 
 	/* Language and file-name stuff */
 	dict->string_set = string_set_create();
-	dict->lang = lang;
 	t = strrchr (lang, '/');
-	if (t) dict->lang = string_set_add(t+1, dict->string_set);
+	t = (NULL == t) ? lang : t+1;
+	dict->lang = string_set_add(t, dict->string_set);
+#ifdef _DEBUG
+	prt_error("Info: Language: %s\n", dict->lang);
+#endif
 	dict->name = string_set_add(dict_name, dict->string_set);
 
 	/*
