@@ -14,8 +14,9 @@
 #include <limits.h>  /* For UINT_MAX */
 
 #include "count.h"
-#include "fast-match.h"
 #include "extract-links.h"
+#include "fast-match.h"
+#include "linkage.h"
 #include "word-utils.h"
 
 /**
@@ -66,11 +67,13 @@ make_choice(Parse_set *lset, int llw, int lrw, Connector * llc, Connector * lrc,
 	pc = (Parse_choice *) xalloc(sizeof(*pc));
 	pc->next = NULL;
 	pc->set[0] = lset;
+	pc->link[0].link_name = NULL;
 	pc->link[0].lw = llw;
 	pc->link[0].rw = lrw;
 	pc->link[0].lc = llc;
 	pc->link[0].rc = lrc;
 	pc->set[1] = rset;
+	pc->link[1].link_name = NULL;
 	pc->link[1].lw = rlw;
 	pc->link[1].rw = rrw;
 	pc->link[1].lc = rlc;
@@ -115,22 +118,8 @@ Parse_info parse_info_new(int nwords)
 	memset(pi, 0, sizeof(struct Parse_info_struct));
 	pi->N_words = nwords;
 
-	/* very unlikely that more than this many links will be needed...*/
-	pi->lasz = 2 * nwords;
-	pi->link_array = (Link*) xalloc(pi->lasz * sizeof(Link));
 	pi->parse_set = NULL;
 
-	pi->chosen_disjuncts = (Disjunct **) xalloc(nwords * sizeof(Disjunct *));
-	memset(pi->chosen_disjuncts, 0, nwords * sizeof(Disjunct *));
-
-#ifdef USE_FAT_LINKAGES
-	pi->image_array = (Image_node **) xalloc(nwords * sizeof(Image_node *));
-	memset(pi->image_array, 0, nwords * sizeof(Image_node *));
-
-	pi->has_fat_down = (char *) xalloc(nwords * sizeof(bool));
-	memset(pi->has_fat_down, 0, nwords * sizeof(bool));
-#endif /* USE_FAT_LINKAGES */
-
 	/* Alloc the x_table */
 	if (nwords >= 10) {
 		log2_table_size = 14;
@@ -157,17 +146,8 @@ Parse_info parse_info_new(int nwords)
 void free_parse_info(Parse_info pi)
 {
 	unsigned int i;
-	int len;
 	X_table_connector *t, *x;
-
-	xfree(pi->link_array, pi->lasz * sizeof(Link));
-
-	len = pi->N_words;
-	xfree(pi->chosen_disjuncts, len * sizeof(Disjunct *));
-#ifdef USE_FAT_LINKAGES
-	xfree(pi->image_array, len * sizeof(Image_node*));
-	xfree(pi->has_fat_down, len * sizeof(bool));
-#endif /* USE_FAT_LINKAGES */
+	if (!pi) return;
 
 	for (i=0; i<pi->x_table_size; i++)
 	{
@@ -363,8 +343,8 @@ Parse_set * mk_parse_set(Sentence sent, fast_matcher_t *mchxt,
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
 
-				Lmatch = (le != NULL) && (d->left != NULL) && do_match(ctxt, le, d->left, lw, w);
-				Rmatch = (d->right != NULL) && (re != NULL) && do_match(ctxt, d->right, re, w, rw);
+				Lmatch = (le != NULL) && (d->left != NULL) && do_match(le, d->left, lw, w);
+				Rmatch = (d->right != NULL) && (re != NULL) && do_match(d->right, re, w, rw);
 				for (i=0; i<4; i++) {ls[i] = rs[i] = NULL;}
 				if (Lmatch)
 				{
@@ -513,88 +493,27 @@ bool build_parse_set(Sentence sent, fast_matcher_t *mchxt,
 	return verify_set(sent->parse_info);
 }
 
-static void initialize_links(Parse_info pi)
+static void issue_link(Linkage lkg, Disjunct * ld, Disjunct * rd, Link * link)
 {
-	pi->N_links = 0;
-	memset(pi->chosen_disjuncts, 0, pi->N_words * sizeof(Disjunct *));
+	check_link_size(lkg);
+	lkg->link_array[lkg->num_links] = *link;
+	lkg->num_links++;
+
+	lkg->chosen_disjuncts[link->lw] = ld;
+	lkg->chosen_disjuncts[link->rw] = rd;
 }
 
-static void issue_link(Parse_info pi, Disjunct * ld, Disjunct * rd, Link link)
-{
-	if (pi->lasz <= pi->N_links)
-	{
-		size_t oldsz = pi->lasz;
-		pi->lasz = 2 * pi->lasz + 10;
-		pi->link_array = xrealloc(pi->link_array, oldsz * sizeof(Link), pi->lasz * sizeof(Link));
-	}
-	pi->link_array[pi->N_links] = link;
-	pi->N_links++;
-
-	pi->chosen_disjuncts[link.lw] = ld;
-	pi->chosen_disjuncts[link.rw] = rd;
-}
-
-static void issue_links_for_choice(Parse_info pi, Parse_choice *pc)
+static void issue_links_for_choice(Linkage lkg, Parse_choice *pc)
 {
 	if (pc->link[0].lc != NULL) { /* there is a link to generate */
-		issue_link(pi, pc->ld, pc->md, pc->link[0]);
+		issue_link(lkg, pc->ld, pc->md, &pc->link[0]);
 	}
 	if (pc->link[1].lc != NULL) {
-		issue_link(pi, pc->md, pc->rd, pc->link[1]);
+		issue_link(lkg, pc->md, pc->rd, &pc->link[1]);
 	}
 }
 
-#ifdef NOT_USED_ANYWHERE
-static void build_current_linkage_recursive(Parse_info pi, Parse_set *set)
-{
-	if (set == NULL) return;
-	if (set->current == NULL) return;
-
-	issue_links_for_choice(pi, set->current);
-	build_current_linkage_recursive(pi, set->current->set[0]);
-	build_current_linkage_recursive(pi, set->current->set[1]);
-}
-
-/**
- * This function takes the "current" point in the given set and
- * generates the linkage that it represents.
- */
-void build_current_linkage(Parse_info pi)
-{
-	initialize_links(pi);
-	build_current_linkage_recursive(pi, pi->parse_set);
-}
-
-/**
- * Advance the "current" linkage to the next one
- * return 1 if there's a "carry" from this node,
- * which indicates that the scan of this node has
- * just been completed, and it's now back to it's
- * starting state.
- */
-static int advance_linkage(Parse_info pi, Parse_set * set)
-{
-	if (set == NULL) return 1;  /* probably can't happen */
-	if (set->first == NULL) return 1;  /* the empty set */
-	if (advance_linkage(pi, set->current->set[0]) == 1) {
-		if (advance_linkage(pi, set->current->set[1]) == 1) {
-			if (set->current->next == NULL) {
-				set->current = set->first;
-				return 1;
-			}
-			set->current = set->current->next;
-		}
-	}
-	return 0;
-}
-
-static void advance_parse_set(Parse_info pi)
-{
-	 advance_linkage(pi, pi->parse_set);
-}
-#endif
-
-static void list_links(Parse_info pi, Parse_set * set, int index)
+static void list_links(Linkage lkg, Parse_set * set, int index)
 {
 	 Parse_choice *pc;
 	 s64 n;
@@ -606,12 +525,12 @@ static void list_links(Parse_info pi, Parse_set * set, int index)
 		  index -= n;
 	 }
 	 assert(pc != NULL, "walked off the end in list_links");
-	 issue_links_for_choice(pi, pc);
-	 list_links(pi, pc->set[0], index % pc->set[0]->count);
-	 list_links(pi, pc->set[1], index / pc->set[0]->count);
+	 issue_links_for_choice(lkg, pc);
+	 list_links(lkg, pc->set[0], index % pc->set[0]->count);
+	 list_links(lkg, pc->set[1], index / pc->set[0]->count);
 }
 
-static void list_random_links(Parse_info pi, Parse_set * set)
+static void list_random_links(Linkage lkg, Parse_info pi, Parse_set * set)
 {
 	Parse_choice *pc;
 	int num_pc, new_index;
@@ -631,9 +550,9 @@ static void list_random_links(Parse_info pi, Parse_set * set)
 	}
 
 	assert(pc != NULL, "Couldn't get a random parse choice");
-	issue_links_for_choice(pi, pc);
-	list_random_links(pi, pc->set[0]);
-	list_random_links(pi, pc->set[1]);
+	issue_links_for_choice(lkg, pc);
+	list_random_links(lkg, pi, pc->set[0]);
+	list_random_links(lkg, pi, pc->set[1]);
 }
 
 /**
@@ -641,14 +560,14 @@ static void list_random_links(Parse_info pi, Parse_set * set)
  * sentence.  For this to work, you must have already called parse, and
  * already built the whole_set.
  */
-void extract_links(int index, Parse_info pi)
+void extract_links(Linkage lkg, Parse_info pi)
 {
-	initialize_links(pi);
+	int index = lkg->lifo.index;
 	if (index < 0) {
 		pi->rand_state = index;
-		list_random_links(pi, pi->parse_set);
+		list_random_links(lkg, pi, pi->parse_set);
 	}
 	else {
-		list_links(pi, pi->parse_set, index);
+		list_links(lkg, pi->parse_set, index);
 	}
 }
