@@ -10,7 +10,6 @@
 #include "matrix-ut.hpp"
 #include "fast-sprintf.hpp"
 
-
 #define  MAX_VARIABLE_NAME 256
 
 
@@ -189,7 +188,9 @@ public:
    */
 
   // If guiding params are unknown, they are set do default
-  int link(int wi, int pi, const char* ci, int wj, int pj, const char* cj) {
+  int link(int wi, int pi, const char* ci, Exp* ei,
+           int wj, int pj, const char* cj, Exp* ej)
+  {
     assert(wi < wj, "Variables: link should be ordered");
     int var;
     if (!get_link_variable(wi, pi, wj, pj, var)) {
@@ -197,7 +198,7 @@ public:
       var_defs_stream << "link_" << wi << "_" << pi << "_" << ci << "_"
                       << wj << "_" << pj << "_" << cj << "\t" << var << endl;
 #endif
-      add_link_variable(wi, pi, ci, wj, pj, cj, var);
+      add_link_variable(wi, pi, ci, ei, wj, pj, cj, ej, var);
       _guiding->setLinkParameters(var, wi, ci, wj, cj, _link_variables[var]->label);
     }
     assert(var != -1, "Var == -1");
@@ -206,11 +207,12 @@ public:
 
   // If the cost is specified, guiding params are calculated
   // using the cost. Any guiding params that are set earlier are overridden
-  int link_cost(int wi, int pi, const char* ci, int wj, int pj, const char* cj,
+  int link_cost(int wi, int pi, const char* ci, Exp* ei,
+                int wj, int pj, const char* cj, Exp* ej,
                 double cost)
   {
     assert(wi < wj, "Variables: link should be ordered");
-    int var = link(wi, pi, ci, wj, pj, cj);
+    int var = link(wi, pi, ci, ei,  wj, pj, cj, ej);
     _guiding->setLinkParameters(var, wi, ci, wj, cj, link_variable(var)->label, cost);
     assert(var != -1, "Var == -1");
     return var;
@@ -326,13 +328,18 @@ public:
   }
 
   // Additional info about the link(wi, pi, wj, pj) variable
-  struct LinkVar {
+  struct LinkVar
+  {
     LinkVar(const std::string& _name, char* _label,
-            int _lw, int _lp, int _rw, int _rp, const char* _lc, const char* _rc)
+            int _lw, int _lp, int _rw, int _rp,
+            const char* _lc, const char* _rc,
+            Exp* _le, Exp* _re)
       : name(_name), label(_label),
-        left_word(_lw), right_word(_rw), left_position(_lp), right_position(_rp),
-        left_connector(_lc), right_connector(_rc) {
-    }
+        left_word(_lw), right_word(_rw),
+        left_position(_lp), right_position(_rp),
+        left_connector(_lc), right_connector(_rc),
+        left_exp(_le), right_exp(_re)
+    {}
 
     std::string name;
     char* label;
@@ -342,6 +349,8 @@ public:
     int right_position;
     const char* left_connector;
     const char* right_connector;
+    Exp* left_exp;
+    Exp* right_exp;
   };
 
   // Returns additional info about the given link variable
@@ -433,7 +442,8 @@ private:
   std::vector<LinkVar*> _link_variables;
 
   // Set this additional info
-  void add_link_variable(int i, int pi, const char* ci, int j, int pj, const char* cj, size_t var)
+  void add_link_variable(int i, int pi, const char* ci, Exp* ei,
+                         int j, int pj, const char* cj, Exp* ej, size_t var)
   {
     char name[MAX_VARIABLE_NAME];
     char* s = name;
@@ -455,16 +465,14 @@ private:
     if (var >= _link_variables.size()) {
       _link_variables.resize(var + 1, 0);
     }
-    _link_variables[var] = new LinkVar(name, label, i, pi, j, pj, ci, cj);
+    _link_variables[var] = new LinkVar(name, label, i, pi, j, pj, ci, cj, ei, ej);
     _link_variables_indices.push_back(var);
   }
 
-
   /*
    * Information about linked(i, j) variables
    */
 
-
   // What is the number of the linked(i, j) variable?
   MatrixUpperTriangle<int> _linked_variable_map;
 
