@@ -5,7 +5,8 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
                                 bool& leading_right, bool& leading_left,
                                 std::vector<int>& eps_right,
                                 std::vector<int>& eps_left,
-                                char* var, bool root, double parent_cost)
+                                char* var, bool root, double parent_cost,
+                                Exp* parent_exp)
 {
   double cost = parent_cost + exp->cost;
   if (exp->type == CONNECTOR_type) {
@@ -18,22 +19,23 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
     connector->string = name;
     set_connector_length_limit(connector);
 
-
-    switch(exp->dir) {
+    switch (exp->dir) {
     case '+':
       _position.push_back(_right_connectors.size());
       _dir.push_back('+');
-      _right_connectors.push_back(PositionConnector(connector, '+', _word, dfs_position, exp->cost, cost,
-                                                    leading_right, false,
-                                                    eps_right, eps_left));
+      _right_connectors.push_back(
+           PositionConnector(parent_exp, connector, '+', _word, dfs_position,
+                             exp->cost, cost, leading_right, false,
+                             eps_right, eps_left));
       leading_right = false;
       break;
     case '-':
       _position.push_back(_left_connectors.size());
       _dir.push_back('-');
-      _left_connectors.push_back(PositionConnector(connector, '-', _word, dfs_position, exp->cost, cost,
-                                                   false, leading_left,
-                                                   eps_right, eps_left));
+      _left_connectors.push_back(
+           PositionConnector(parent_exp, connector, '-', _word, dfs_position,
+                             exp->cost, cost, false, leading_left,
+                             eps_right, eps_left));
       leading_left = false;
       break;
     default:
@@ -45,7 +47,8 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
     } else
       if (exp->u.l != NULL && exp->u.l->next == NULL) {
         /* unary and - skip */
-        insert_connectors(exp->u.l->e, dfs_position, leading_right, leading_left, eps_right, eps_left, var, root, cost);
+        insert_connectors(exp->u.l->e, dfs_position, leading_right,
+             leading_left, eps_right, eps_left, var, root, cost, parent_exp);
       } else {
         int i;
         E_list* l;
@@ -63,7 +66,8 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
           *s++ = 'c';
           fast_sprintf(s, i);
 
-          insert_connectors(l->e, dfs_position, leading_right, leading_left, eps_right, eps_left, new_var, false, cost);
+          insert_connectors(l->e, dfs_position, leading_right, leading_left,
+                eps_right, eps_left, new_var, false, cost, parent_exp);
           if (leading_right && var != NULL) {
             eps_right.push_back(_variables->epsilon(new_var, '+'));
           }
@@ -77,7 +81,8 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
   } else if (exp->type == OR_type) {
     if (exp->u.l != NULL && exp->u.l->next == NULL) {
       /* unary or - skip */
-      insert_connectors(exp->u.l->e, dfs_position, leading_right, leading_left, eps_right, eps_left, var, root, cost);
+      insert_connectors(exp->u.l->e, dfs_position, leading_right, leading_left,
+          eps_right, eps_left, var, root, cost, exp->u.l->e);
     } else {
       int i;
       E_list* l;
@@ -100,7 +105,7 @@ void WordTag::insert_connectors(Exp* exp, int& dfs_position,
         *s++ = 'd';
         fast_sprintf(s, i);
 
-        insert_connectors(l->e, dfs_position, lr, ll, er, el, new_var, false, cost);
+        insert_connectors(l->e, dfs_position, lr, ll, er, el, new_var, false, cost, l->e);
         if (lr)
           lr_true = true;
         if (ll)
