@@ -9,10 +9,22 @@
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
 /*************************************************************************/
+#ifndef _STRING_SET_H_
+#define _STRING_SET_H_
 
+#include <stddef.h>
 #include "api-types.h"
 
+struct String_set_s
+{
+   size_t size;       /* the current size of the table */
+   size_t count;      /* number of things currently in the table */
+   char ** table;     /* the table itself */
+};
+
 String_set * string_set_create(void);
 const char * string_set_add(const char * source_string, String_set * ss);
 const char * string_set_lookup(const char * source_string, String_set * ss);
 void         string_set_delete(String_set *ss);
+
+#endif /* _STRING_SET_H_ */
