@@ -9,9 +9,9 @@
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
 /*************************************************************************/
-Boolean separate_sentence(Sentence, Parse_Options);
+bool separate_sentence(Sentence, Parse_Options);
 void build_sentence_expressions(Sentence, Parse_Options);
-Boolean sentence_in_dictionary(Sentence);
+bool sentence_in_dictionary(Sentence);
 #ifdef USE_ANYSPLIT
 void add_alternative(Sentence sent,
                      int prefnum, const char **prefix,
