@@ -36,25 +36,13 @@ void free_X_nodes(X_node *);
 Connector * connector_new(void);
 Connector * init_connector(Connector *c);
 void free_connectors(Connector *);
-void exfree_connectors(Connector *);
-Connector * copy_connectors(Connector *);
-Connector * excopy_connectors(Connector * c);
-
-/* Link utilities ... */
-Link *      excopy_link(Link *);
-void        exfree_link(Link *);
-
 
 /* Connector-set utilities ... */
 Connector_set * connector_set_create(Exp *e);
 void connector_set_delete(Connector_set * conset);
 bool word_has_connector(Dict_node *, const char *, char);
 const char * word_only_connector(Dict_node *);
-#ifdef USE_FAT_LINKAGES
-bool match_in_connector_set(count_context_t*, Connector_set*, Connector*, int);
-#else
 bool match_in_connector_set(Connector_set*, Connector*, int dir);
-#endif
 
 
 /**
@@ -92,13 +80,7 @@ static inline bool easy_match(const char * s, const char * t)
 
 	while ((*s!='\0') && (*t!='\0'))
 	{
-		if ((*s == '*') || (*t == '*') ||
-#ifdef USE_FAT_LINKAGES
-			((*s == *t) && (*s != '^'))
-#else
-			(*s == *t)
-#endif
-		)
+		if ((*s == '*') || (*t == '*') || (*s == *t))
 		{
 			s++;
 			t++;
